@@ -1,8 +1,11 @@
-from e3.event import EventHandler
-from e3.fs import mkdir, cp
+from __future__ import annotations
+
 import json
 import os
 
+from e3.event import EventHandler
+from e3.fs import cp, mkdir
+
 
 class FileHandler(EventHandler):
     def __init__(self, log_dir):
@@ -10,7 +13,7 @@ class FileHandler(EventHandler):
 
     def send_event(self, event):
         d = event.as_dict()
-        prefix = "%s-%s" % (d["name"], d["uid"])
+        prefix = "{}-{}".format(d["name"], d["uid"])
         log_file = os.path.join(self.log_dir, prefix + ".log")
         attach_dir = os.path.join(self.log_dir, prefix)
         mkdir(attach_dir)
@@ -21,7 +24,7 @@ class FileHandler(EventHandler):
         return True
 
     @classmethod
-    def decode_config(self, config_str):
+    def decode_config(cls, config_str):
         return {"log_dir": config_str}
 
     def encode_config(self):
