@@ -1,7 +1,10 @@
-from e3.event import EventHandler
+from __future__ import annotations
+
 import json
 import logging
+
 import e3.log
+from e3.event import EventHandler
 
 
 class LoggingHandler(EventHandler):
@@ -20,4 +23,4 @@ class LoggingHandler(EventHandler):
         return {"logger_name": logger_name, "level": level}
 
     def encode_config(self):
-        return "%s,%s" % (self.logger_name, self.level)
+        return f"{self.logger_name},{self.level}"
