@@ -1,16 +1,15 @@
+from __future__ import annotations
+
 import base64
-import calendar
 import json
-from datetime import datetime
+from e3.date import utc_timestamp
 
 
-def get_payload(token):
+def get_payload(token: str) -> dict:
     """Get payload from a JSON Web Token.
 
     :param token: token
-    :type token: str
     :return: decoded payload
-    :rtype: dict
     """
     data = {}
     if token.count(".") == 2:
@@ -22,29 +21,17 @@ def get_payload(token):
         if rem > 0:
             payload += "=" * (4 - rem)
         try:
-            data = json.loads(base64.b64decode(payload).decode("utf-8"))
-        except (UnicodeDecodeError, ValueError, TypeError):
+            data = json.loads(base64.urlsafe_b64decode(payload).decode("utf-8"))
+        except (ValueError, TypeError):
             pass
     return data
 
 
-def utc_timestamp():
-    """Return the number of seconds since epoch UTC.
-
-    :return: the number of seconds since epoch UTC
-    :rtype: int
-    """
-    d = datetime.utcnow()
-    return calendar.timegm(d.utctimetuple())
-
-
-def is_valid(token):
+def is_valid(token: str) -> bool:
     """Return true if the token is still valid, false otherwise.
 
     :param token: full token
-    :type token: str
     :return: return true if the token is still valid, false otherwise
-    :rtype: bool
     """
     payload = get_payload(token)
 
