@@ -7,6 +7,8 @@ status
 """
 
 
+from __future__ import annotations
+
 import errno
 import logging
 import os
@@ -14,10 +16,31 @@ import signal
 import subprocess
 import sys
 import time
+from typing import TYPE_CHECKING
 
 import e3.env
 import e3.log
 from e3.os.fs import which
+from e3.text import bytes_as_str
+
+
+if TYPE_CHECKING:
+    from typing import cast, Any, IO, List, Literal, NoReturn, Optional, Union
+
+    CmdLine = List[str]
+    AnyCmdLine = Union[List[CmdLine], CmdLine]
+    STDOUT_VALUE = Literal[-1]
+    PIPE_VALUE = Literal[-2]
+    DEVNULL_VALUE = Literal[-3]
+
+    # Make STDOUT subprocess constant visible in e3.os.process
+    STDOUT = cast(STDOUT_VALUE, subprocess.STDOUT)
+    PIPE = cast(PIPE_VALUE, subprocess.PIPE)
+    DEVNULL = cast(DEVNULL_VALUE, subprocess.DEVNULL)
+else:
+    STDOUT = subprocess.STDOUT
+    PIPE = subprocess.PIPE
+    DEVNULL = subprocess.DEVNULL
 
 logger = e3.log.getLogger("os.process")
 
@@ -27,9 +50,6 @@ CMD_LOGGER_NAME = "os.process.cmdline"
 
 cmdlogger = e3.log.getLogger(CMD_LOGGER_NAME)
 
-# Make STDOUT subprocess constant visible in e3.os.process
-STDOUT = subprocess.STDOUT
-PIPE = subprocess.PIPE
 
 # Use psutil.Popen when available to get psutil.Process properties and
 # methods available in Run.internal
@@ -42,7 +62,7 @@ except ImportError:  # defensive code
     psutil = None
 
 
-def subprocess_setup():
+def subprocess_setup() -> None:
     """Reset SIGPIPE handler.
 
     Python installs a SIGPIPE handler by default. This is usually not
@@ -53,23 +73,24 @@ def subprocess_setup():
     signal.signal(signal.SIGPIPE, signal.SIG_DFL)  # all: no cover
 
 
-def get_rlimit(platform=None):
+def get_rlimit(platform: Optional[str] = None) -> str:
     if platform is None:
         platform = e3.env.Env().build.platform
+    if platform == "x86_64-windows64":
+        platform = "x86_64-windows"
 
     from pkg_resources import resource_filename
 
     return resource_filename(__name__, os.path.join("data", "rlimit-%s" % platform))
 
 
-def quote_arg(arg):
+def quote_arg(arg: str) -> str:
     """Return the quoted version of the given argument.
 
     Returns a human-friendly representation of the given argument, but with all
     extra quoting done if necessary.  The intent is to produce an argument
     image that can be copy/pasted on a POSIX shell command (at a shell prompt).
     :param arg: argument to quote
-    :type arg: str
     """
     # The empty argument is a bit of a special case, as it does not
     # contain any character that might need quoting, and yet still
@@ -123,13 +144,20 @@ def quote_arg(arg):
     return arg
 
 
-def command_line_image(cmds):
+def to_cmd_lines(cmds: AnyCmdLine) -> List[CmdLine]:
+    if isinstance(cmds[0], str):
+        # Turn the simple command into a special case of
+        # the multiple-commands case.  This will allow us
+        # to treat both cases the same way.
+        return [cmds]  # type: ignore
+    else:
+        return cmds  # type: ignore
+
+
+def command_line_image(cmds: AnyCmdLine) -> str:
     """Return a string image of the given command(s).
 
     :param cmds: Same as the cmds parameter in the Run.__init__ method.
-    :type: list[str] | list[list[str]]
-
-    :rtype: str
 
     This method also handles quoting as defined for POSIX shells.
     This means that arguments containing special characters
@@ -141,29 +169,23 @@ def command_line_image(cmds):
     The result is expected to be a string that can be sent verbatim
     to a shell for execution.
     """
-    if isinstance(cmds[0], str):
-        # Turn the simple command into a special case of
-        # the multiple-commands case.  This will allow us
-        # to treat both cases the same way.
-        cmds = (cmds,)
-    return " | ".join((" ".join((quote_arg(arg) for arg in cmd)) for cmd in cmds))
+    return " | ".join(
+        " ".join(quote_arg(arg) for arg in cmd) for cmd in to_cmd_lines(cmds)
+    )
 
 
-def enable_commands_handler(filename, mode="a"):
+def enable_commands_handler(filename: str, mode: str = "a") -> logging.Handler:
     """Add a handler that log all commands launched with Run in a file.
 
     :param filename: path to log the commands
-    :type filename: str
     :param mode: mode used to open the file (default is 'a')
-    :type mode: str
     :return: the added handler
-    :type: logging.Handler
     """
 
     class CmdFilter(logging.Filter):
         """Keep only e3.os.process.cmdline records."""
 
-        def filter(self, record):
+        def filter(self, record: logging.LogRecord) -> int:
             return 1 if record.name == "e3." + CMD_LOGGER_NAME else 0
 
     # Here we don't attach the handler directly to the cmdline logger. Indeed
@@ -177,18 +199,17 @@ def enable_commands_handler(filename, mode="a"):
     return fh
 
 
-def disable_commands_handler(handler):
+def disable_commands_handler(handler: logging.Handler) -> None:
     """Disable special handler for commands.
 
     :param handler: Handler returned by enable_commands_handler
-    :type handler: logging.Handler
     """
     logging.getLogger().removeHandler(handler)
     handler.flush()
     handler.close()
 
 
-class Run(object):
+class Run:
     """Class to handle processes.
 
     :ivar cmds: The ``cmds`` argument passed to the __init__ method
@@ -198,25 +219,26 @@ class Run(object):
         the process has exited, its initial value is None.  When a problem
         running the command is detected and a process does not get
         created, its value gets set to the special value 127.
-    :ivar out: process standard output  (if instanciated with output = PIPE)
-    :ivar err: same as out but for standard error
+    :ivar raw_out: process standard output as bytes (if instanciated with
+        output = PIPE). Use self.out to get a decoded string.
+    :ivar raw_err: same as raw_out but for standard error.
     :ivar pid: PID. Set to -1 if the command failed to run.
     """
 
     def __init__(
         self,
-        cmds,
-        cwd=None,
-        output=PIPE,
-        error=STDOUT,
-        input=None,
-        bg=False,
-        timeout=None,
-        env=None,
-        set_sigpipe=True,
-        parse_shebang=False,
-        ignore_environ=True,
-    ):
+        cmds: AnyCmdLine,
+        cwd: Optional[str] = None,
+        output: Union[STDOUT_VALUE, DEVNULL_VALUE, PIPE_VALUE, str, IO, None] = PIPE,
+        error: Union[STDOUT_VALUE, DEVNULL_VALUE, PIPE_VALUE, str, IO, None] = STDOUT,
+        input: Union[DEVNULL_VALUE, PIPE_VALUE, str, IO, None] = None,
+        bg: bool = False,
+        timeout: Optional[int] = None,
+        env: Optional[dict] = None,
+        set_sigpipe: bool = True,
+        parse_shebang: bool = False,
+        ignore_environ: bool = True,
+    ) -> None:
         """Spawn a process.
 
         :param cmds: two possibilities:
@@ -226,37 +248,26 @@ class Run(object):
             different commands will be piped. This means that
             [['ps', '-a'], ['grep', 'vxsim']] will be equivalent to
             the system command line 'ps -a | grep vxsim'.
-        :type cmds: list[str] | list[list[str]]
         :param cwd: directory in which the process should be executed (string
             or None). If None then current directory is used
-        :type cwd: str | None
         :param output: can be PIPE (default), a filename string, a fd on an
             already opened file, a python file object or None (for stdout).
-        :type output: int | str | file | None
         :param error: same as output or STDOUT, which indicates that the
             stderr data from the applications should be captured into the same
             file handle as for stdout.
-        :type error: int | str | file | None
         :param input: same as output
-        :type input: int | str | file | None
         :param bg: if True then run in background
-        :type bg: bool
         :param timeout: limit execution time (in seconds), None means
             unlimited
-        :type timeout: int | None
         :param env: dictionary for environment variables (e.g. os.environ)
-        :type env: dict
         :param set_sigpipe: reset SIGPIPE handler to default value
-        :type set_sigpipe: bool
         :param parse_shebang: take the #! interpreter line into account
-        :type parse_shebang: bool
         :param ignore_environ: Applies only when env parameter is not None.
             When set to True (the default), the only environment variables
             passed to the program are the ones provided by the env parameter.
             Otherwise, the environment passed to the program consists of the
             environment variables currently defined (os.environ) augmented by
             the ones provided in env.
-        :type ignore_environ: bool
 
         :raise OSError: when trying to execute a non-existent file.
 
@@ -267,7 +278,7 @@ class Run(object):
         will be used for process stdin.
         """
 
-        def add_interpreter_command(cmd_line):
+        def add_interpreter_command(cmd_line: CmdLine) -> CmdLine:
             """Add the interpreter defined in the #! line to cmd_line.
 
             If the #! line cannot be parsed, just return the cmd_line
@@ -276,7 +287,6 @@ class Run(object):
             On windows, /usr/bin/env will be ignored to avoid a dependency on
             cygwin and /bin/bash & /bin/sh are replaced by $SHELL if defined.
             :param cmd_line: command line
-            :type cmd_line: list[str]
             """
             if not parse_shebang:
                 # nothing to do
@@ -299,7 +309,7 @@ class Run(object):
                 f.seek(0)
                 line = f.readline()
                 interpreter_cmds = [
-                    l.strip() for l in line[line.find("!") + 1 :].split()
+                    word.strip() for word in line[line.find("!") + 1 :].split()
                 ]
                 # Pass the program path to the interpreter
                 if len(cmd_line) > 1:
@@ -330,9 +340,9 @@ class Run(object):
         self.output_file = File(output, "w")
         self.error_file = File(error, "w")
 
-        self.status = None
-        self.out = ""
-        self.err = ""
+        self.status: Optional[int] = None
+        self.raw_out = b""
+        self.raw_err = b""
         self.cmds = []
 
         if env is not None:
@@ -351,7 +361,7 @@ class Run(object):
                     env = tmp
             else:
                 # ignore_environ is False, so get a copy of the current
-                # environment and update it with the env dictionnary.
+                # environment and update it with the env dictionary.
                 tmp = os.environ.copy()
                 tmp.update(env)
                 env = tmp
@@ -366,11 +376,8 @@ class Run(object):
                 rlimit_args = []
 
         try:
-            if isinstance(cmds[0], str):
-                self.cmds = rlimit_args + list(add_interpreter_command(cmds))
-            else:
-                self.cmds = [add_interpreter_command(c) for c in cmds]
-                self.cmds[0] = rlimit_args + list(self.cmds[0])
+            self.cmds = [add_interpreter_command(c) for c in to_cmd_lines(cmds)]
+            self.cmds[0] = rlimit_args + list(self.cmds[0])
 
             cmdlogger.debug(
                 "Run: cd %s; %s",
@@ -378,27 +385,27 @@ class Run(object):
                 self.command_line_image(),
             )
 
-            if isinstance(cmds[0], str):
+            if len(self.cmds) == 1:
                 popen_args = {
                     "stdin": self.input_file.fd,
                     "stdout": self.output_file.fd,
                     "stderr": self.error_file.fd,
                     "cwd": cwd,
                     "env": env,
-                    "universal_newlines": True,
+                    "universal_newlines": False,
                 }
 
                 if sys.platform != "win32" and set_sigpipe:  # windows: no cover
                     # preexec_fn is no supported on windows
-                    popen_args["preexec_fn"] = subprocess_setup
+                    popen_args["preexec_fn"] = subprocess_setup  # type: ignore
 
                 if sys.platform == "win32":
                     popen_args["creationflags"] = subprocess.CREATE_NEW_PROCESS_GROUP
 
-                self.internal = Popen(self.cmds, **popen_args)
+                self.internal = Popen(self.cmds[0], **popen_args)
 
             else:
-                runs = []
+                runs: List[Popen] = []
                 for index, cmd in enumerate(self.cmds):
                     if index == 0:
                         stdin = self.input_file.fd
@@ -411,10 +418,8 @@ class Run(object):
                     # (e.g. gzip -dc foo.txt | tar -xf -)
                     if index == len(self.cmds) - 1:
                         stdout = self.output_file.fd
-                        txt_mode = True
                     else:
                         stdout = subprocess.PIPE
-                        txt_mode = False
 
                     popen_args = {
                         "stdin": stdin,
@@ -422,12 +427,12 @@ class Run(object):
                         "stderr": self.error_file.fd,
                         "cwd": cwd,
                         "env": env,
-                        "universal_newlines": txt_mode,
+                        "universal_newlines": False,
                     }
 
                     if sys.platform != "win32" and set_sigpipe:  # windows: no cover
                         # preexec_fn is no supported on windows
-                        popen_args["preexec_fn"] = subprocess_setup
+                        popen_args["preexec_fn"] = subprocess_setup  # type: ignore
 
                     if sys.platform == "win32":
                         popen_args[
@@ -455,32 +460,49 @@ class Run(object):
         if not bg:
             self.wait()
 
-    def command_line_image(self):
-        """Get shell command line image of the spawned command(s).
+    @property
+    def out(self) -> str:
+        """Process output as string.
 
-        :rtype: str
+        Attempt is done to decode as utf-8 the output. If the output is not in
+        utf-8 a string representation will be returned
+        (see e3.text.bytes_as_str).
+        """
+        return bytes_as_str(self.raw_out)
+
+    @property
+    def err(self) -> str:
+        """Process error as string.
+
+        Attempt is done to decode as utf-8 the output. If the output is not in
+        utf-8 a string representation will be returned
+        (see e3.text.bytes_as_str).
+        """
+        return bytes_as_str(self.raw_err)
+
+    def command_line_image(self) -> str:
+        """Get shell command line image of the spawned command(s).
 
         This just a convenient wrapper around the function of the same
         name.
         """
         return command_line_image(self.cmds)
 
-    def close_files(self):
+    def close_files(self) -> None:
         """Close all file descriptors."""
         self.output_file.close()
         self.error_file.close()
         self.input_file.close()
 
-    def __error(self, error, cmds):
+    def __error(self, error: Exception, cmds: List[CmdLine]) -> None:
         """Set pid to -1 and status to 127 before closing files."""
         self.close_files()
         logger.error(error)
 
-        def not_found(path):
+        def not_found(path: str) -> NoReturn:
             """Raise OSError.
 
             :param path: path of the executable
-            :type path: str
             """
             logger.error("%s not found", path)
             e3.log.debug("PATH=%s", os.environ["PATH"])
@@ -490,19 +512,14 @@ class Run(object):
 
         # Try to send an helpful message if one of the executable has not
         # been found.
-        if isinstance(cmds[0], str):
-            if which(cmds[0], default=None) is None:
-                not_found(cmds[0])
-        else:
-            for cmd in cmds:
-                if which(cmd[0], default=None) is None:
-                    not_found(cmd[0])
+        for cmd in cmds:
+            if which(cmd[0], default=None) is None:
+                not_found(cmd[0])
 
-    def wait(self):
+    def wait(self) -> int:
         """Wait until process ends and return its status.
 
         :return: exit code of the process
-        :rtype: int
         """
         if self.status is not None:
             # Wait has already been called
@@ -518,17 +535,20 @@ class Run(object):
         ):
             self.status = self.internal.wait()
         else:
-            tmp_input = None
+            tmp_input: Optional[Union[str, bytes]] = None
             if self.input_file.fd == subprocess.PIPE:
                 tmp_input = self.input_file.get_command()
 
-            (self.out, self.err) = self.internal.communicate(tmp_input)
+            if isinstance(tmp_input, str):
+                tmp_input = tmp_input.encode("utf-8")
+
+            (self.raw_out, self.raw_err) = self.internal.communicate(tmp_input)
             self.status = self.internal.returncode
 
         self.close_files()
         return self.status
 
-    def poll(self):
+    def poll(self) -> Optional[int]:
         """Check the process status and set self.status if available.
 
         This method checks whether the underlying process has exited
@@ -538,7 +558,6 @@ class Run(object):
 
         :return: None if the process is still alive; otherwise, returns
           the process exit status.
-        :rtype: int | None
         """
         if self.status is not None:
             # Process is already terminated and wait been called
@@ -553,60 +572,53 @@ class Run(object):
         else:
             return None
 
-    def kill(self, recursive=True, timeout=3):
+    def kill(self, recursive: bool = True, timeout: int = 3) -> None:
         """Kill the process.
 
         :param recursive: if True, try to kill the complete process tree
-        :type recursive: bool
         :param timeout: wait timeout (in seconds) after sending the kill
             signal (when recursive=True)
-        :type timeout: int
         """
         if recursive:
             kill_process_tree(self.internal, timeout=timeout)
         else:
             self.internal.kill()
 
-    def interrupt(self):
+    def interrupt(self) -> None:
         """Send SIGINT to the process, kill on Windows."""
         if sys.platform == "win32":
             self.kill()  # Ctrl-C event is unreliable on Windows
         else:
             self.internal.send_signal(signal.SIGINT)
 
-    def is_running(self):
-        """Check whether the process is running.
-
-        :rtype: bool
-        """
+    def is_running(self) -> bool:
+        """Check whether the process is running."""
         if psutil is None:  # defensive code
             # psutil not imported, use our is_running function
             return is_running(self.pid)
         else:
             return self.internal.is_running()
 
-    def children(self):
-        """Return list of child processes (using psutil).
-
-        :rtype: list[psutil.Process]
-        """
+    def children(self) -> List[psutil.Process]:
+        """Return list of child processes (using psutil)."""
         if psutil is None:  # defensive code
             raise NotImplementedError("Run.children() require psutil")
         return self.internal.children()
 
 
-class File(object):
+class File:
     """Can be a PIPE, a file object."""
 
-    def __init__(self, name, mode="r"):
+    def __init__(self, name: Any, mode: str = "r"):
         """Create a new File.
 
         :param name: can be PIPE, STDOUT, a filename string, an opened fd, a
             python file object, or a command to pipe (if starts with |)
         :param mode: can be 'r' or 'w' if name starts with + the mode will be
-            a+ :type mode: str
+            a+
         """
         assert mode in "rw", "Mode should be r or w"
+        self.fd: Union[int, IO[str]]
 
         self.name = name
         self.to_close = False
@@ -633,32 +645,32 @@ class File(object):
             # this is a file descriptor
             self.fd = name
 
-    def get_command(self):
+    def get_command(self) -> Optional[str]:
         """Return the command to run to create the pipe."""
         if self.fd == subprocess.PIPE:
             return self.name[1:]
+        else:
+            return None
 
-    def close(self):
+    def close(self) -> None:
         """Close the file if needed."""
         if self.to_close:
-            self.fd.close()
+            fd = self.fd
+            fd.close()  # type: ignore
 
 
 class WaitError(Exception):
     pass
 
 
-def wait_for_processes(process_list, timeout):
+def wait_for_processes(process_list: List[Run], timeout: float) -> Optional[int]:
     """Wait for several processes spawned with Run.
 
     :param process_list: a list of Run objects
-    :type process_list: list[Run]
     :param timeout: a timeout in seconds. If 0 block until a process ends.
-    :type timeout: int
 
     :return: None in case of timeout or the index in process Run corresponding
         to the first process that end
-    :rtype: None | int
     """
     if len(process_list) == 0:
         return None
@@ -698,7 +710,7 @@ def wait_for_processes(process_list, timeout):
         # then select which support timeout arguments to wait.
         fd_r, fd_w = os.pipe()
 
-        def handler(signum, frame):
+        def handler(signum: int, frame: Any) -> None:
             del signum, frame
             os.write(fd_w, b"a")
 
@@ -720,11 +732,11 @@ def wait_for_processes(process_list, timeout):
 
                 while True:
                     try:
-                        l_r, _, _ = select.select(*select_args)
+                        l_r, _, _ = select.select(*select_args)  # type: ignore
                         if l_r:
                             os.read(fd_r, 1)
                         break
-                    except select.error:
+                    except OSError:
                         pass
 
                 remain = timeout - time.time() + start
@@ -738,15 +750,13 @@ def wait_for_processes(process_list, timeout):
             signal.signal(signal.SIGCHLD, 0)
             os.close(fd_r)
             os.close(fd_w)
+    return None
 
 
-def is_running(pid):
+def is_running(pid: int) -> bool:
     """Check whether a process with the given pid is running.
 
     :param pid: an integer (e.g the value of Run().pid)
-    :type pid: int
-
-    :rtype: bool
     """
     if sys.platform == "win32":  # unix: no cover
         from e3.os.windows.native_api import Access, NT
@@ -772,16 +782,13 @@ def is_running(pid):
         return True
 
 
-def kill_process_tree(pid, timeout=3):
+def kill_process_tree(pid: Union[int, psutil.Process], timeout: int = 3) -> bool:
     """Kill a hierarchy of processes.
 
     :param pid: pid of the toplevel process
-    :type pid: int | psutil.Process
     :param timeout: wait timeout after sending the kill signal
-    :type timeout: int
     :return: True if all processes either don't exist or have been killed,
         False if there are some processes still alive.
-    :rtype: bool
     """
     if isinstance(pid, psutil.Process):
         parent_process = pid
@@ -808,7 +815,7 @@ def kill_process_tree(pid, timeout=3):
         except psutil.NoSuchProcess:  # defensive code
             pass
 
-    def on_terminate(p):
+    def on_terminate(p: str) -> None:
         """Log info when a process terminate."""
         logger.info("process %s killed", p)
 
