@@ -1,4 +1,5 @@
 import os
+import sys
 
 import e3.anod.helper
 import e3.anod.sandbox
@@ -99,7 +100,7 @@ def test_deploy_sandbox():
         fd.write("        pass\n")
 
     assert (
-        "cannot get resource metadata from store"
+        "no download metadata returned by the download primitive"
         in e3.os.process.Run([os.path.join("bin", "anod"), "download", "b"]).out
     )
 
@@ -382,7 +383,7 @@ def test_sandbox_source_auto_ignore(git_specs_dir):
     assert "found 2" in p.out
     assert "found 1" in p.out
     for subdir in ("3", "3/2", "3/2/1"):
-        assert "found .anod in {}".format(subdir) in p.out
+        assert f"found .anod in {subdir}" in p.out
     assert "result: OK" in p.out
 
 
@@ -599,6 +600,7 @@ def test_sandbox_user_yaml(git_specs_dir):
     p = e3.os.process.Run(
         [
             "e3-sandbox",
+            "-v",
             "exec",
             "--plan",
             os.path.join(sandbox_dir, "test.plan"),
@@ -606,10 +608,14 @@ def test_sandbox_user_yaml(git_specs_dir):
         ]
     )
     assert "build dummyspec" in p.out
-    assert "using alternate specs dir {}".format(local_spec_dir) in p.out
+    assert f"using alternate specs dir {local_spec_dir}" in p.out
 
     sbx = e3.anod.sandbox.SandBox()
     sbx.root_dir = sandbox_dir
 
     assert sbx.is_alternate_specs_dir
-    assert sbx.specs_dir == local_spec_dir
+
+    if sys.platform == "win32":
+        assert sbx.specs_dir.lower() == local_spec_dir.lower()
+    else:
+        assert sbx.specs_dir == local_spec_dir
