@@ -59,7 +59,7 @@ def setup_sbx(request):
     os.mkdir(SBX_TMP_DIR)
 
 
-def job_source_basename(uid):
+def job_source_basename(uid: str) -> str:
     """Return the name of a source corresponding to the give job.
 
     In our testcase environment, we will consider that, if a DAG's action
@@ -74,30 +74,24 @@ def job_source_basename(uid):
     naming scheme (see DOWNLOAD_JOB_UID_PREFIX above).
 
     :param uid: A unique Job ID.
-    :type uid: str
-    :rtype: str
     """
     if uid.startswith(DOWNLOAD_JOB_UID_PREFIX):
         uid = uid[len(DOWNLOAD_JOB_UID_PREFIX) :]
     return uid + ".txt"
 
 
-def source_fullpath(uid):
+def source_fullpath(uid: str) -> str:
     """Return the fullpath of a job's sources, if present.
 
     :param uid: A unique Job ID.
-    :type uid: str
-    :rtype: str
     """
     return os.path.join(SBX_TMP_DIR, job_source_basename(uid))
 
 
-def source_store_fullpath(uid):
+def source_store_fullpath(uid: str) -> str:
     """Return the given job's fullpath to its sources in the store.
 
     :param uid: A unique Job ID.
-    :type uid: str
-    :rtype: str
     """
     assert uid.startswith(DOWNLOAD_JOB_UID_PREFIX)
     return os.path.join(STORE_DIR, job_source_basename(uid))
@@ -127,12 +121,12 @@ class ControlledJob(ProcessJob):
     """
 
     def __init__(self, uid, data, notify_end):
-        super(ControlledJob, self).__init__(uid, data, notify_end)
+        super().__init__(uid, data, notify_end)
         self.run_count = 0
 
     def run(self):
         self.run_count += 1
-        return super(ControlledJob, self).run()
+        return super().run()
 
     @property
     def cmdline(self):
@@ -160,7 +154,7 @@ class SimpleWalk(Walk):
         # The list of jobs (by UID) that have been requeued.
         self.saved_jobs = {}
         self.requeued = {}
-        super(SimpleWalk, self).__init__(actions)
+        super().__init__(actions)
 
     def request_requeue(self, job):
         """Requeue the job is not already queued once."""
@@ -186,7 +180,7 @@ class SimpleWalk(Walk):
         # as a way to record some information each time this method
         # is called, so as to be able to verify some aspects of
         # the class' behavior.
-        job = super(SimpleWalk, self).get_job(uid, data, predecessors, notify_end)
+        job = super().get_job(uid, data, predecessors, notify_end)
         self.saved_jobs[job.uid] = job
         return job
 
@@ -243,7 +237,7 @@ class FingerprintWalkDryRun(FingerprintWalk):
 
 
 @pytest.mark.parametrize("walk_class", [SimpleWalk, FingerprintWalk])
-class TestWalk(object):
+class TestWalk:
     def test_good_job_no_predecessors(self, walk_class, setup_sbx):
         """Simple case of a leaf job."""
         actions = DAG()
